package cn.iocoder.yudao.module.system.service.config;

import cn.iocoder.yudao.module.system.controller.admin.config.vo.*;
import cn.iocoder.yudao.module.system.dal.dataobject.config.CommonConfigDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskDetailDO;
import cn.iocoder.yudao.module.system.dal.mysql.config.CommonConfigMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;


import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.CONFIG_NOT_EXISTS;

/**
 * 通用配置 Service 实现类
 *
 * @author 陈宾顺
 */
@Service
@Validated
public class CommonConfigServiceImpl implements CommonConfigService {

    @Resource
    private CommonConfigMapper configMapper;

    @Override
    public Long createConfig(CommonConfigSaveReqVO createReqVO) {
        // 插入
        CommonConfigDO config = BeanUtils.toBean(createReqVO, CommonConfigDO.class);
        configMapper.insert(config);
        // 返回
        return config.getId();
    }

    @Override
    public void updateConfig(CommonConfigSaveReqVO updateReqVO) {
        // 校验存在
        validateConfigExists(updateReqVO.getId());
        // 更新
        CommonConfigDO updateObj = BeanUtils.toBean(updateReqVO, CommonConfigDO.class);
        configMapper.updateById(updateObj);
    }

    @Override
    public void deleteConfig(Long id) {
        // 校验存在
        validateConfigExists(id);
        // 删除
        configMapper.deleteById(id);
    }

    private void validateConfigExists(Long id) {
        if (configMapper.selectById(id) == null) {
            throw exception(CONFIG_NOT_EXISTS);
        }
    }

    @Override
    public CommonConfigDO getConfig(Long configType) {
        CommonConfigDO commonConfigDO = configMapper.selectOne(new LambdaQueryWrapper<CommonConfigDO>()
                .eq(CommonConfigDO::getConfigType, configType));
        return commonConfigDO;
    }

    @Override
    public PageResult<CommonConfigDO> getConfigPage(CommonConfigPageReqVO pageReqVO) {
        return configMapper.selectPage(pageReqVO);
    }

}