package cn.iocoder.yudao.module.system.service.houselocation;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.houselocation.vo.WareHouseLocationPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.houselocation.vo.WareHouseLocationRespVO;
import cn.iocoder.yudao.module.system.controller.admin.houselocation.vo.WareHouseLocationSaveReqVO;
import cn.iocoder.yudao.module.system.controller.admin.houselocation.vo.WareHouseLocationVO;
import cn.iocoder.yudao.module.system.dal.dataobject.houselocation.WareHouseLocationDO;
import com.baomidou.mybatisplus.extension.service.IService;

import javax.validation.Valid;
import java.util.List;

/**
 * 库位 Service 接口
 *
 * @author 陈宾顺
 */
public interface HouseLocationService extends IService<WareHouseLocationDO> {

    /**
     * 创建库位
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createHouseLocation(@Valid WareHouseLocationSaveReqVO createReqVO);

    /**
     * 更新库位
     *
     * @param updateReqVO 更新信息
     */
    void updateHouseLocation(@Valid WareHouseLocationSaveReqVO updateReqVO);

    /**
     * 删除库位
     *
     * @param id 编号
     */
    void deleteHouseLocation(Long id);

    /**
     * 获得库位
     *
     * @param id 编号
     * @return 库位
     */
    WareHouseLocationDO getHouseLocation(Long id);

    /**
     * 获得库位分页
     *
     * @param pageReqVO 分页查询
     * @return 库位分页
     */
    PageResult<WareHouseLocationDO> getHouseLocationPage(WareHouseLocationPageReqVO pageReqVO);

    /**
     * 库位批量保存或修改或删除
     *
     * @param positionMapId
     * @param list
     */
    void batchSaveOrEditOrDel(Long positionMapId, List<List<WareHouseLocationDO>> list);

    /**
     * 根据地图查询库位
     *
     * @param positionMapId
     * @return
     */
    List<WareHouseLocationDO> getByMapId(Long positionMapId);

    /**
     * @param requestVO
     * @return
     */
    List<WareHouseLocationRespVO> getLocationByName(WareHouseLocationVO requestVO);

    /**
     * 根据地图id 点位id 获取库位信息
     *
     * @param mapId
     * @param mapItemId
     * @return
     */
    List<WareHouseLocationDO> getByMapItemId(Long mapId, Long mapItemId);
}
