package cn.iocoder.yudao.module.system.service.information;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.information.dto.DeviceInformationDTO;
import cn.iocoder.yudao.module.system.controller.admin.information.dto.MapBindDeviceInfoDTO;
import cn.iocoder.yudao.module.system.controller.admin.information.dto.StatisticsInformationDTO;
import cn.iocoder.yudao.module.system.controller.admin.information.vo.DeviceInformationPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.information.vo.DeviceInformationSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.information.DeviceInformationDO;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * 设备信息 Service 接口
 *
 * @author 陈宾顺
 */
public interface DeviceInformationService {

    /**
     * 创建设备信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createInformation(@Valid DeviceInformationSaveReqVO createReqVO);

    /**
     * 更新设备信息
     *
     * @param updateReqVO 更新信息
     */
    void updateInformation(@Valid DeviceInformationSaveReqVO updateReqVO);

    /**
     * 删除设备信息
     *
     * @param id 编号
     */
    void deleteInformation(Long id);

    /**
     * 获得设备信息
     *
     * @param id 编号
     * @return 设备信息
     */
    DeviceInformationDO getInformation(Long id);

    /**
     * 获得设备信息分页
     *
     * @param pageReqVO 分页查询
     * @return 设备信息分页
     */
    PageResult<DeviceInformationDO> getInformationPage(DeviceInformationPageReqVO pageReqVO);

    /**
     * 批量新增删除修改
     *
     * @param positionMapId
     * @param list
     */
    void batchSaveOrEditOrDel(Long positionMapId, List<List<DeviceInformationDO>> list);

    /**
     * 通过地图id获取设备列表
     *
     * @param positionMapId
     * @return
     */
    List<DeviceInformationDO> getByMapId(Long positionMapId);

    /**
     * 获取设备列表
     *
     * @param dto
     * @return
     */
    List<DeviceInformationDO> getList(@Valid DeviceInformationDTO dto);

    /**
     * 地图绑定设备
     *
     * @param dto
     */
    void mapBindDeviceInfo(@Valid MapBindDeviceInfoDTO dto);

    /**
     * 获得设备信息分类列表
     * @return
     */
    Map<Integer, List<DeviceInformationDO>> classification();

    /**
     * 统计设备
     * @return
     */
    List<StatisticsInformationDTO> getDeviceNumber();

    /**
     * 查看全部设备
     * @param pageReqVO
     * @return
     */
    List<DeviceInformationDO> getInformationList(@Valid DeviceInformationPageReqVO pageReqVO);
}
