package cn.iocoder.yudao.module.system.service.path;

import cn.hutool.core.util.ObjectUtil;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.mqtt.api.path.PathPlanningApi;
import cn.iocoder.yudao.module.mqtt.api.path.dto.PositionMapLineDTO;
import cn.iocoder.yudao.module.mqtt.api.path.dto.RelatedPathNode;
import cn.iocoder.yudao.module.system.constant.path.PathPlanningTopicConstant;
import cn.iocoder.yudao.module.system.controller.admin.positionmap.vo.PositionMapSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.positionmap.PositionMapDO;
import cn.iocoder.yudao.module.system.dal.dataobject.positionmap.PositionMapLineDO;
import cn.iocoder.yudao.module.system.dal.mysql.positionmap.PositionMapLineMapper;
import cn.iocoder.yudao.module.system.dal.mysql.positionmap.PositionMapMapper;
import cn.iocoder.yudao.module.system.enums.line.DirectionEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.PATH_PLANNING_NOT_EXISTS;

/**
 * 同步信息给车辆
 */
@Slf4j
@Service
public class PathPlanningServiceImpl implements PathPlanningService {


    @Resource
    private PositionMapLineMapper positionMapLineMapper;

    @Resource
    private PositionMapMapper positionMapMapper;

    public static ExecutorService executor = Executors.newFixedThreadPool(5);

    @Resource
    private PathPlanningApi pathPlanningApi;

    /**
     * 同步全部的点位信息
     */
    @Override
    public void synchronousPoint(PositionMapSaveReqVO data) {

        List<PositionMapDO> positionMapDOS = positionMapMapper.selectList(new LambdaQueryWrapperX<PositionMapDO>()
        .eq(ObjectUtil.isNotEmpty(data.getFloor()), PositionMapDO::getFloor, data.getFloor())
        .eq(ObjectUtil.isNotEmpty(data.getArea()), PositionMapDO::getArea, data.getArea()));

        PositionMapLineDO positionMapLineDO = new PositionMapLineDO();
        List<PositionMapLineDTO> positionLines = positionMapLineMapper.getAllPositionMapLine(positionMapLineDO);

//        List<PositionMapLineDO> positionLines = positionMapLineMapper.selectList(new LambdaQueryWrapperX<PositionMapLineDO>());
        if (ObjectUtil.isEmpty(positionMapDOS) || ObjectUtil.isEmpty(positionLines)) {
            throw exception(PATH_PLANNING_NOT_EXISTS);
        }

        Map<Long, List<PositionMapLineDTO>> positionMapLineMap = positionLines.stream()
                .collect(Collectors.groupingBy(PositionMapLineDTO::getPositionMapId));
        for (PositionMapDO positionMapDO : positionMapDOS) {
            RelatedPathNode relatedPathNode = new RelatedPathNode();
            relatedPathNode.setFloor(positionMapDO.getFloor());
            relatedPathNode.setArea(positionMapDO.getArea());
            List<PositionMapLineDTO> positionMapLineDOS = positionMapLineMap.get(positionMapDO.getId());
            if (ObjectUtil.isEmpty(positionMapLineDOS)) {
                continue;
            }

            assembleDataToPP(relatedPathNode,positionMapLineDOS);
        }

    }

    /**
     * 组装数据发送到PP
     * @param relatedPathNode
     * @param positionMapLineDOS
     */
    private void assembleDataToPP(RelatedPathNode relatedPathNode, List<PositionMapLineDTO> positionMapLineDOS) {
        executor.execute(() -> {
            List<PositionMapLineDTO> list = new ArrayList<>();
            for (PositionMapLineDTO positionMapLineDO : positionMapLineDOS) {
                list.add(positionMapLineDO);
                if (DirectionEnum.ONE_WAY.getType().equals(positionMapLineDO.getDirection())) {
                    continue;
                }

                PositionMapLineDTO build = PositionMapLineDTO.builder()
                        .id(positionMapLineDO.getId())
                        .positionMapId(positionMapLineDO.getPositionMapId())
                        .startingPointId(positionMapLineDO.getEndPointId())
                        .endPointId(positionMapLineDO.getStartingPointId())
                        .direction(1)
                        .forwardSpeedLimit(positionMapLineDO.getReverseSpeedLimit())
                        .reverseSpeedLimit(positionMapLineDO.getForwardSpeedLimit())
                        .beginControlX(positionMapLineDO.getEndControlX())
                        .beginControlY(positionMapLineDO.getEndControlY())
                        .endControlX(positionMapLineDO.getBeginControlX())
                        .endControlY(positionMapLineDO.getBeginControlY())
                        .expansionZoneFront(positionMapLineDO.getExpansionZoneAfter())
                        .expansionZoneAfter(positionMapLineDO.getExpansionZoneFront())
                        .expansionZoneLeft(positionMapLineDO.getExpansionZoneRight())
                        .expansionZoneRight(positionMapLineDO.getExpansionZoneLeft())
                        .toward(positionMapLineDO.getToward())
                        .build();
                list.add(build);
            }

            relatedPathNode.setControl_nodes(list);

            pathPlanningApi.synchronousPointToPP(relatedPathNode, PathPlanningTopicConstant.SYNCHRONOUS_ALL_MAP_POINT);
        });
    }































}
