package cn.iocoder.yudao.module.system.service.positionmap;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.system.controller.admin.positionmap.dto.PositionMapConditionDTO;
import cn.iocoder.yudao.module.system.controller.admin.positionmap.vo.PositionMapItemPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.positionmap.vo.PositionMapItemSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.positionmap.PositionMapItemDO;
import cn.iocoder.yudao.module.system.dal.mysql.positionmap.PositionMapItemMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;

import static cn.hutool.core.collection.CollUtil.isNotEmpty;
import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertList;

/**
 * 仓库点位地图子表 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class PositionMapItemServiceImpl extends ServiceImpl<PositionMapItemMapper, PositionMapItemDO> implements PositionMapItemService {

    @Resource
    private PositionMapItemMapper positionMapItemMapper;

    @Override
    public Long createPositionMapItem(PositionMapItemSaveReqVO createReqVO) {
        // 插入
        PositionMapItemDO positionMapItem = BeanUtils.toBean(createReqVO, PositionMapItemDO.class);
        positionMapItemMapper.insert(positionMapItem);
        // 返回
        return positionMapItem.getId();
    }

    @Override
    public void updatePositionMapItem(PositionMapItemSaveReqVO updateReqVO) {
        // 更新
        PositionMapItemDO updateObj = BeanUtils.toBean(updateReqVO, PositionMapItemDO.class);
        positionMapItemMapper.updateById(updateObj);
    }

    @Override
    public void deletePositionMapItem(Long id) {
        // 删除
        positionMapItemMapper.deleteById(id);
    }

    @Override
    public PositionMapItemDO getPositionMapItem(Long id) {
        return positionMapItemMapper.selectById(id);
    }

    @Override
    public PageResult<PositionMapItemDO> getPositionMapItemPage(PositionMapItemPageReqVO pageReqVO) {
        return positionMapItemMapper.selectPage(pageReqVO);
    }

    @Override
    public List<PositionMapItemDO> getByMapId(Long positionMapId) {
        return positionMapItemMapper.selectList(new LambdaQueryWrapperX<PositionMapItemDO>()
                .eq(PositionMapItemDO::getPositionMapId, positionMapId));
    }

    @Override
    public void batchSaveOrEditOrDel(Long positionMapId, List<List<PositionMapItemDO>> list) {
        //批量添加、修改、删除
        if (isNotEmpty(list.get(0))) {
            list.get(0).forEach(a -> a.setPositionMapId(positionMapId));
            positionMapItemMapper.insertBatch(list.get(0));
        }
        if (isNotEmpty(list.get(1))) {
            list.get(1).forEach(a -> a.setPositionMapId(positionMapId));
            positionMapItemMapper.updateBatch(list.get(1));
        }
        if (isNotEmpty(list.get(2))) {
            positionMapItemMapper.deleteByIds(convertList(list.get(2), PositionMapItemDO::getId));
        }
    }


    @Override
    public List<PositionMapItemDO> getByCondition(PositionMapConditionDTO dto) {
        return positionMapItemMapper.selectList(new LambdaQueryWrapperX<PositionMapItemDO>()
                .eqIfPresent(PositionMapItemDO::getPositionMapId, dto.getPositionMapId())
                .eqIfPresent(PositionMapItemDO::getType, dto.getType())
                .inIfPresent(PositionMapItemDO::getId, dto.getIds())
                .eqIfPresent(PositionMapItemDO::getAreaId, dto.getAreaId())
                .eqIfPresent(PositionMapItemDO::getLaneId, dto.getLaneId()));
    }


}
