package cn.iocoder.yudao.module.system.service.robot;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.*;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotInformationDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 车辆信息 Service 接口
 *
 * @author 陈宾顺
 */
public interface RobotInformationService {

    /**
     * 创建车辆信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createInformation(@Valid RobotInformationSaveReqVO createReqVO);

    /**
     * 更新车辆信息
     *
     * @param updateReqVO 更新信息
     */
    void updateInformation(@Valid RobotInformationSaveReqVO updateReqVO);

    /**
     * 删除车辆信息
     *
     * @param id 编号
     */
    void deleteInformation(Long id);

    /**
     * 获得车辆信息
     *
     * @param id 编号
     * @return 车辆信息
     */
    RobotInformationRespVO getInformation(Long id);

    /**
     * 获得车辆信息分页
     *
     * @param pageReqVO 分页查询
     * @return 车辆信息分页
     */
    PageResult<RobotInformationPageRespVO> getInformationPage(RobotInformationPageReqVO pageReqVO);

    /**
     * 统计车辆待命/任务中/离线
     *
     * @return
     */
    RobotInformationStatisticsVO statisticsInformation();

    /**
     * 查询能正常使用的车辆
     *
     * @return
     */
    List<RobotInformationDO> getCanUseRobot();

    /**
     * 查询所有车辆
     *
     * @return
     */
    List<RobotInformationDO> getAllRobot();

    /**
     * 查询机器人编号
     *
     * @param mac
     * @return
     */
    String getRobotNoByMac(String mac);
}
